@@ -19,6 +19,11 @@ try:
 except ImportError:
     php = None
 
+try:
+    from sphinx_csharp import csharp as cs  # type: ignore
+except ImportError:
+    cs = None
+
 import re
 import six
 import textwrap
@@ -140,6 +145,55 @@ class PyClasslike(BaseObject, python.PyClasslike):
     pass
 
 
+# ----------------------------------------------------------------------------
+
+if cs is not None:
+    class CSharpCurrentNamespace(BaseObject, cs.CSharpCurrentNamespace):
+        pass
+
+    class CSharpNamespacePlain(BaseObject, cs.CSharpNamespacePlain):
+        pass
+
+    class CSharpClass(BaseObject, cs.CSharpClass):
+        pass
+
+    class CSharpStruct(BaseObject, cs.CSharpStruct):
+        pass
+
+    class CSharpInterface(BaseObject, cs.CSharpInterface):
+        pass
+
+    class CSharpInherits(BaseObject, cs.CSharpInherits):
+        pass
+
+    class CSharpMethod(BaseObject, cs.CSharpMethod):
+        pass
+
+    class CSharpVariable(BaseObject, cs.CSharpVariable):
+        pass
+
+    class CSharpProperty(BaseObject, cs.CSharpProperty):
+        pass
+
+    class CSharpEvent(BaseObject, cs.CSharpEvent):
+        pass
+
+    class CSharpEnum(BaseObject, cs.CSharpEnum):
+        pass
+
+    class CSharpEnumValue(BaseObject, cs.CSharpEnumValue):
+        pass
+
+    class CSharpAttribute(BaseObject, cs.CSharpAttribute):
+        pass
+
+    class CSharpIndexer(BaseObject, cs.CSharpIndexer):
+        pass
+
+    class CSharpXRefRole(BaseObject, cs.CSharpXRefRole):
+        pass
+
+
 # ----------------------------------------------------------------------------
 
 class DomainDirectiveFactory:
@@ -191,6 +245,30 @@ class DomainDirectiveFactory:
             'global': (php.PhpGloballevel, 'global'),
         }
 
+    if cs is not None:
+        cs_classes = {
+            # 'doxygen-name': (CSharp class, key in CSharpDomain.object_types)
+            'namespace': (CSharpNamespacePlain, 'namespace'),
+
+            'class': (CSharpClass, 'class'),
+            'struct': (CSharpStruct, 'struct'),
+            'interface': (CSharpInterface, 'interface'),
+
+            'function': (CSharpMethod, 'function'),
+            'method': (CSharpMethod, 'method'),
+
+            'variable': (CSharpVariable, 'var'),
+            'property': (CSharpProperty, 'property'),
+            'event': (CSharpEvent, 'event'),
+
+            'enum': (CSharpEnum, 'enum'),
+            'enumvalue': (CSharpEnumValue, 'enumerator'),
+            'attribute': (CSharpAttribute, 'attr'),
+
+            # Fallback to cpp domain
+            'typedef': (CPPTypeObject, 'type'),
+        }
+
     @staticmethod
     def create(domain: str, args) -> ObjectDescription:
         cls = cast(Type[ObjectDescription], None)
@@ -214,6 +292,8 @@ class DomainDirectiveFactory:
                     arg_0 = 'global'
             cls, name = DomainDirectiveFactory.php_classes.get(
                 arg_0, (php.PhpClasslike, 'class'))
+        elif cs is not None and domain == 'cs':
+            cls, name = DomainDirectiveFactory.cs_classes[args[0]]  # type: ignore
         else:
             domain = 'cpp'
             cls, name = DomainDirectiveFactory.cpp_classes[args[0]]  # type: ignore
@@ -446,11 +526,12 @@ class SphinxRenderer:
         declaration = declaration.replace('\n', ' ')
         nodes_ = self.run_directive(obj_type, declaration, content_callback, options)
         if self.app.env.config.breathe_debug_trace_doxygen_ids:
-            ts = self.create_doxygen_target(node)
-            if len(ts) == 0:
+            target = self.create_doxygen_target(node)
+            if len(target) == 0:
                 print("{}Doxygen target: (none)".format('  ' * _debug_indent))
             else:
-                print("{}Doxygen target: {}".format('  ' * _debug_indent, ts[0]['ids']))
+                print("{}Doxygen target: {}".format('  ' * _debug_indent,
+                                                    target[0]['ids']))
 
         # <desc><desc_signature> and then one or more <desc_signature_line>
         # each <desc_signature_line> has a sphinx_line_type which hints what is present in that line
@@ -477,7 +558,8 @@ class SphinxRenderer:
             assert n.astext()[-1] == " "
             txt = display_obj_type + ' '
             declarator[0] = addnodes.desc_annotation(txt, txt)
-        target = self.create_doxygen_target(node)
+        if not self.app.env.config.breathe_debug_trace_doxygen_ids:
+            target = self.create_doxygen_target(node)
         declarator.insert(0, target)
         if declarator_callback:
             declarator_callback(declarator)
@@ -644,11 +726,11 @@ class SphinxRenderer:
             obj_type = node.kind
         nodes = self.run_domain_directive(obj_type, [declaration.replace('\n', ' ')])
         if self.app.env.config.breathe_debug_trace_doxygen_ids:
-            ts = self.create_doxygen_target(node)
-            if len(ts) == 0:
+            target = self.create_doxygen_target(node)
+            if len(target) == 0:
                 print("{}Doxygen target (old): (none)".format('  ' * _debug_indent))
             else:
-                print("{}Doxygen target (old): {}".format('  ' * _debug_indent, ts[0]['ids']))
+                print("{}Doxygen target (old): {}".format('  ' * _debug_indent, target[0]['ids']))
 
         rst_node = nodes[1]
         finder = NodeFinder(rst_node.document)
@@ -662,7 +744,9 @@ class SphinxRenderer:
             update_signature(signode, obj_type)
         if description is None:
             description = self.description(node)
-        signode.insert(0, self.create_doxygen_target(node))
+        if not self.app.env.config.breathe_debug_trace_doxygen_ids:
+            target = self.create_doxygen_target(node)
+        signode.insert(0, target)
         contentnode.extend(description)
         return nodes
 
@@ -827,11 +911,11 @@ class SphinxRenderer:
             return self.visit_union(node)
         elif kind in ('struct', 'class', 'interface'):
             dom = self.get_domain()
-            if not dom or dom in ('c', 'cpp', 'py'):
+            if not dom or dom in ('c', 'cpp', 'py', 'cs'):
                 return self.visit_class(node)
         elif kind == 'namespace':
             dom = self.get_domain()
-            if not dom or dom in ('c', 'cpp', 'py'):
+            if not dom or dom in ('c', 'cpp', 'py', 'cs'):
                 return self.visit_namespace(node)
 
         self.context = cast(RenderContext, self.context)
@@ -924,7 +1008,7 @@ class SphinxRenderer:
     # having two lists in case they fall out of sync
     #
     # If this list is edited, also change the sections option documentation for
-    # the doxygen(auto)file directive in documentation/source/autofile.rst.
+    # the doxygen(auto)file directive in documentation/source/file.rst.
     sections = [
         ("user-defined", "User Defined"),
         ("public-type", "Public Types"),
@@ -933,8 +1017,8 @@ class SphinxRenderer:
         ("public-slot", "Public Slots"),
         ("signal", "Signals"),
         ("dcop-func", "DCOP Function"),
-        ("property", "Property"),
-        ("event", "Event"),
+        ("property", "Properties"),
+        ("event", "Events"),
         ("public-static-func", "Public Static Functions"),
         ("public-static-attrib", "Public Static Attributes"),
         ("protected-type", "Protected Types"),
@@ -1024,6 +1108,12 @@ class SphinxRenderer:
         addnode('innerclass', lambda: self.render_iterable(node.innerclass))
         addnode('innernamespace', lambda: self.render_iterable(node.innernamespace))
 
+        if 'inner' in options:
+            for node in node.innergroup:
+                file_data = self.compound_parser.parse(node.refid)
+                inner = file_data.compounddef
+                addnode('innergroup', lambda: self.visit_compounddef(inner))
+
         nodelist = []
         for i, nodes_ in sorted(nodemap.items()):
             nodelist += nodes_
@@ -1344,7 +1434,7 @@ class SphinxRenderer:
 
     def visit_function(self, node) -> List[Node]:
         dom = self.get_domain()
-        if not dom or dom in ('c', 'cpp', 'py'):
+        if not dom or dom in ('c', 'cpp', 'py', 'cs'):
             names = self.get_qualification()
             names.append(node.get_name())
             name = self.join_nested_name(names)
@@ -1396,11 +1486,12 @@ class SphinxRenderer:
 
             nodes = self.run_domain_directive(node.kind, self.context.directive_args[1])
             if self.app.env.config.breathe_debug_trace_doxygen_ids:
-                ts = self.create_doxygen_target(node)
-                if len(ts) == 0:
+                target = self.create_doxygen_target(node)
+                if len(target) == 0:
                     print("{}Doxygen target (old): (none)".format('  ' * _debug_indent))
                 else:
-                    print("{}Doxygen target (old): {}".format('  ' * _debug_indent, ts[0]['ids']))
+                    print("{}Doxygen target (old): {}".format('  ' * _debug_indent,
+                                                              target[0]['ids']))
 
             rst_node = nodes[1]
             finder = NodeFinder(rst_node.document)
@@ -1408,7 +1499,9 @@ class SphinxRenderer:
 
             # Templates have multiple signature nodes in recent versions of Sphinx.
             # Insert Doxygen target into the first signature node.
-            rst_node.children[0].insert(0, self.create_doxygen_target(node))  # type: ignore
+            if not self.app.env.config.breathe_debug_trace_doxygen_ids:
+                target = self.create_doxygen_target(node)
+            rst_node.children[0].insert(0, target)  # type: ignore
 
             finder.content.extend(self.description(node))
             return nodes
@@ -1502,7 +1595,7 @@ class SphinxRenderer:
                 node.get_argsstring(),
                 self.make_initializer(node)
             ])
-        if not dom or dom in ('c', 'cpp', 'py'):
+        if not dom or dom in ('c', 'cpp', 'py', 'cs'):
             return self.handle_declaration(node, declaration, options=options)
         else:
             return self.render_declaration(node, declaration)
@@ -1647,6 +1740,12 @@ class SphinxRenderer:
             return self.visit_typedef(node)
         if node.kind == "variable":
             return self.visit_variable(node)
+        if node.kind == "property":
+            # Note: visit like variable for now
+            return self.visit_variable(node)
+        if node.kind == "event":
+            # Note: visit like variable for now
+            return self.visit_variable(node)
         if node.kind == "define":
             return self.visit_define(node)
         if node.kind == "friend":
